<?php

namespace Litepie\Deal\Policies;

use Litepie\User\Contracts\UserPolicy;
use Litepie\Deal\Models\Medium;

class MediumPolicy
{

    /**
     * Determine if the given user can view the medium.
     *
     * @param UserPolicy $user
     * @param Medium $medium
     *
     * @return bool
     */
    public function view(UserPolicy $user, Medium $medium)
    {
        if ($user->canDo('deal.medium.view') && $user->isAdmin()) {
            return true;
        }

        return $medium->user_id == user_id() && $medium->user_type == user_type();
    }

    /**
     * Determine if the given user can create a medium.
     *
     * @param UserPolicy $user
     * @param Medium $medium
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('deal.medium.create');
    }

    /**
     * Determine if the given user can update the given medium.
     *
     * @param UserPolicy $user
     * @param Medium $medium
     *
     * @return bool
     */
    public function update(UserPolicy $user, Medium $medium)
    {
        if ($user->canDo('deal.medium.edit') && $user->isAdmin()) {
            return true;
        }

        return $medium->user_id == user_id() && $medium->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given medium.
     *
     * @param UserPolicy $user
     * @param Medium $medium
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Medium $medium)
    {
        return $medium->user_id == user_id() && $medium->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given medium.
     *
     * @param UserPolicy $user
     * @param Medium $medium
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Medium $medium)
    {
        if ($user->canDo('deal.medium.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given medium.
     *
     * @param UserPolicy $user
     * @param Medium $medium
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Medium $medium)
    {
        if ($user->canDo('deal.medium.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
