<?php

namespace Litepie\Deal\Providers;

use Illuminate\Support\ServiceProvider;

class DealServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'deal');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'deal');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerDeal();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerDeal()
    {
        $this->app->bind('deal', function($app) {
            return new Deal($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Deal', 'Lavalite\Deal\Facades\Deal');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('litepie.deal', function ($app) {
            return $this->app->make('Litepie\Deal\Deal');
        });

        // Bind Deal to repository
        $this->app->bind(
            'Litepie\Deal\Interfaces\DealRepositoryInterface',
            \Litepie\Deal\Repositories\Eloquent\DealRepository::class
        );
        // Bind Tag to repository
        $this->app->bind(
            'Litepie\Deal\Interfaces\TagRepositoryInterface',
            \Litepie\Deal\Repositories\Eloquent\TagRepository::class
        );
        // Bind Category to repository
        $this->app->bind(
            'Litepie\Deal\Interfaces\CategoryRepositoryInterface',
            \Litepie\Deal\Repositories\Eloquent\CategoryRepository::class
        );
        // Bind Detail to repository
        $this->app->bind(
            'Litepie\Deal\Interfaces\DetailRepositoryInterface',
            \Litepie\Deal\Repositories\Eloquent\DetailRepository::class
        );
        // Bind Medium to repository
        $this->app->bind(
            'Litepie\Deal\Interfaces\MediumRepositoryInterface',
            \Litepie\Deal\Repositories\Eloquent\MediumRepository::class
        );

        $this->app->register(\Litepie\Deal\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Litepie\Deal\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and deal's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'litepie.deal'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeDeal::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['litepie.deal'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('litepie/deal.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/deal')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/deal')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
