<?php

namespace Litepie\Deal\Providers;

use Illuminate\Foundation\Support\Providers\RouteServiceProvider as ServiceProvider;
use Illuminate\Routing\Router;
use Litepie\Deal\Models\Deal;
use Request;
use Route;

class RouteServiceProvider extends ServiceProvider
{
    /**
     * This namespace is applied to the controller routes in your routes file.
     *
     * In addition, it is set as the URL generator's root namespace.
     *
     * @var string
     */
    protected $namespace = 'Litepie\Deal\Http\Controllers';

    /**
     * Define your route model bindings, pattern filters, etc.
     *
     * @param   \Illuminate\Routing\Router  $router
     * @return void
     */
    public function boot()
    {
        parent::boot();

        if (Request::is('*/deal/deal/*')) {
            Route::bind('deal', function ($deal) {
                $dealrepo = $this->app->make('Litepie\Deal\Interfaces\DealRepositoryInterface');
                return $dealrepo->findorNew($deal);
            });
        }

        if (Request::is('*/deal/tag/*')) {
            Route::bind('tag', function ($tag) {
                $tagrepo = $this->app->make('Litepie\Deal\Interfaces\TagRepositoryInterface');
                return $tagrepo->findorNew($tag);
            });
        }

        if (Request::is('*/deal/category/*')) {
            Route::bind('category', function ($category) {
                $categoryrepo = $this->app->make('Litepie\Deal\Interfaces\CategoryRepositoryInterface');
                return $categoryrepo->findorNew($category);
            });
        }

        if (Request::is('*/deal/detail/*')) {
            Route::bind('detail', function ($detail) {
                $detailrepo = $this->app->make('Litepie\Deal\Interfaces\DetailRepositoryInterface');
                return $detailrepo->findorNew($detail);
            });
        }

        if (Request::is('*/deal/medium/*')) {
            Route::bind('medium', function ($medium) {
                $mediumrepo = $this->app->make('Litepie\Deal\Interfaces\MediumRepositoryInterface');
                return $mediumrepo->findorNew($medium);
            });
        }

    }

    /**
     * Define the routes for the package.
     *
     * @return void
     */
    public function map()
    {
        $this->mapWebRoutes();
    }

    /**
     * Define the "web" routes for the package.
     *
     * These routes all receive session state, CSRF protection, etc.
     *
     * @return void
     */
    protected function mapWebRoutes()
    {   
        Route::group([
            'middleware' => 'web',
            'namespace'  => $this->namespace,
        ], function ($router) {
            require (__DIR__ . '/../../routes/web.php');
        });
    }

}
